package App::Toodledo::NotebookInternal;

use Carp;
use Moose;

our $VERSION = '1.00';

with 'App::Toodledo::NotebookRole';

1;

__END__

=head1 NAME

App::Toodledo::NotebookInternal - internal representation of a notebook.

=head1 SYNOPSIS

For internal L<App::Toodledo> use only.

=head1 DESCRIPTION

For internal L<App::Toodledo> use only.

=head1 AUTHOR

Peter Scott C<cpan at psdt.com>

=cut
